@@ -19,6 +19,7 @@ import gitlab
 from gitlab import base
 from gitlab import cli
 from gitlab import exceptions as exc
+from gitlab import types as g_types
 
 
 class GetMixin(object):
@@ -31,7 +32,7 @@ class GetMixin(object):
             lazy (bool): If True, don't request the server, but create a
                          shallow object giving access to the managers. This is
                          useful if you want to avoid useless calls to the API.
-            **kwargs: Extra options to send to the Gitlab server (e.g. sudo)
+            **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
             object: The generated RESTObject.
@@ -55,7 +56,7 @@ class GetWithoutIdMixin(object):
         """Retrieve a single object.
 
         Args:
-            **kwargs: Extra options to send to the Gitlab server (e.g. sudo)
+            **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
             object: The generated RESTObject
@@ -65,9 +66,33 @@ class GetWithoutIdMixin(object):
             GitlabGetError: If the server cannot perform the request
         """
         server_data = self.gitlab.http_get(self.path, **kwargs)
+        if server_data is None:
+            return None
         return self._obj_cls(self, server_data)
 
 
+class RefreshMixin(object):
+    @exc.on_http_error(exc.GitlabGetError)
+    def refresh(self, **kwargs):
+        """Refresh a single object from server.
+
+        Args:
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Returns None (updates the object)
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabGetError: If the server cannot perform the request
+        """
+        if self._id_attr:
+            path = '%s/%s' % (self.manager.path, self.id)
+        else:
+            path = self.manager.path
+        server_data = self.manager.gitlab.http_get(path, **kwargs)
+        self._update_attrs(server_data)
+
+
 class ListMixin(object):
     @exc.on_http_error(exc.GitlabListError)
     def list(self, **kwargs):
@@ -79,7 +104,7 @@ class ListMixin(object):
             page (int): ID of the page to return (starts with page 1)
             as_list (bool): If set to False and no pagination option is
                 defined, return a generator instead of a list
-            **kwargs: Extra options to send to the Gitlab server (e.g. sudo)
+            **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
             list: The list of objects, or a generator if `as_list` is False
@@ -89,43 +114,29 @@ class ListMixin(object):
             GitlabListError: If the server cannot perform the request
         """
 
+        # Duplicate data to avoid messing with what the user sent us
+        data = kwargs.copy()
+        if self.gitlab.per_page:
+            data.setdefault('per_page', self.gitlab.per_page)
+
+        # We get the attributes that need some special transformation
+        types = getattr(self, '_types', {})
+        if types:
+            for attr_name, type_cls in types.items():
+                if attr_name in data.keys():
+                    type_obj = type_cls(data[attr_name])
+                    data[attr_name] = type_obj.get_for_api()
+
         # Allow to overwrite the path, handy for custom listings
-        path = kwargs.pop('path', self.path)
-        obj = self.gitlab.http_list(path, **kwargs)
+        path = data.pop('path', self.path)
+
+        obj = self.gitlab.http_list(path, **data)
         if isinstance(obj, list):
             return [self._obj_cls(self, item) for item in obj]
         else:
             return base.RESTObjectList(self, self._obj_cls, obj)
 
 
-class GetFromListMixin(ListMixin):
-    def get(self, id, **kwargs):
-        """Retrieve a single object.
-
-        Args:
-            id (int or str): ID of the object to retrieve
-            **kwargs: Extra options to send to the Gitlab server (e.g. sudo)
-
-        Returns:
-            object: The generated RESTObject
-
-        Raises:
-            GitlabAuthenticationError: If authentication is not correct
-            GitlabGetError: If the server cannot perform the request
-        """
-        try:
-            gen = self.list()
-        except exc.GitlabListError:
-            raise exc.GitlabGetError(response_code=404,
-                                     error_message="Not found")
-
-        for obj in gen:
-            if str(obj.get_id()) == str(id):
-                return obj
-
-        raise exc.GitlabGetError(response_code=404, error_message="Not found")
-
-
 class RetrieveMixin(ListMixin, GetMixin):
     pass
 
@@ -157,7 +168,7 @@ class CreateMixin(object):
         Args:
             data (dict): parameters to send to the server to create the
                          resource
-            **kwargs: Extra options to send to the Gitlab server (e.g. sudo)
+            **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
             RESTObject: a new instance of the managed object class built with
@@ -168,11 +179,29 @@ class CreateMixin(object):
             GitlabCreateError: If the server cannot perform the request
         """
         self._check_missing_create_attrs(data)
-        if hasattr(self, '_sanitize_data'):
-            data = self._sanitize_data(data, 'create')
+        files = {}
+
+        # We get the attributes that need some special transformation
+        types = getattr(self, '_types', {})
+        if types:
+            # Duplicate data to avoid messing with what the user sent us
+            data = data.copy()
+            for attr_name, type_cls in types.items():
+                if attr_name in data.keys():
+                    type_obj = type_cls(data[attr_name])
+
+                    # if the type if FileAttribute we need to pass the data as
+                    # file
+                    if issubclass(type_cls, g_types.FileAttribute):
+                        k = type_obj.get_file_name(attr_name)
+                        files[attr_name] = (k, data.pop(attr_name))
+                    else:
+                        data[attr_name] = type_obj.get_for_api()
+
         # Handle specific URL for creation
         path = kwargs.pop('path', self.path)
-        server_data = self.gitlab.http_post(path, post_data=data, **kwargs)
+        server_data = self.gitlab.http_post(path, post_data=data, files=files,
+                                            **kwargs)
         return self._obj_cls(self, server_data)
 
 
@@ -196,6 +225,18 @@ class UpdateMixin(object):
         """
         return getattr(self, '_update_attrs', (tuple(), tuple()))
 
+    def _get_update_method(self):
+        """Return the HTTP method to use.
+
+        Returns:
+            object: http_put (default) or http_post
+        """
+        if getattr(self, '_update_uses_post', False):
+            http_method = self.gitlab.http_post
+        else:
+            http_method = self.gitlab.http_put
+        return http_method
+
     @exc.on_http_error(exc.GitlabUpdateError)
     def update(self, id=None, new_data={}, **kwargs):
         """Update an object on the server.
@@ -203,7 +244,7 @@ class UpdateMixin(object):
         Args:
             id: ID of the object to update (can be None if not required)
             new_data: the update data for the object
-            **kwargs: Extra options to send to the Gitlab server (e.g. sudo)
+            **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
             dict: The new object data (*not* a RESTObject)
@@ -219,12 +260,27 @@ class UpdateMixin(object):
             path = '%s/%s' % (self.path, id)
 
         self._check_missing_update_attrs(new_data)
-        if hasattr(self, '_sanitize_data'):
-            data = self._sanitize_data(new_data, 'update')
-        else:
-            data = new_data
+        files = {}
 
-        return self.gitlab.http_put(path, post_data=data, **kwargs)
+        # We get the attributes that need some special transformation
+        types = getattr(self, '_types', {})
+        if types:
+            # Duplicate data to avoid messing with what the user sent us
+            new_data = new_data.copy()
+            for attr_name, type_cls in types.items():
+                if attr_name in new_data.keys():
+                    type_obj = type_cls(new_data[attr_name])
+
+                    # if the type if FileAttribute we need to pass the data as
+                    # file
+                    if issubclass(type_cls, g_types.FileAttribute):
+                        k = type_obj.get_file_name(attr_name)
+                        files[attr_name] = (k, new_data.pop(attr_name))
+                    else:
+                        new_data[attr_name] = type_obj.get_for_api()
+
+        http_method = self._get_update_method()
+        return http_method(path, post_data=new_data, files=files, **kwargs)
 
 
 class SetMixin(object):
@@ -257,15 +313,18 @@ class DeleteMixin(object):
 
         Args:
             id: ID of the object to delete
-            **kwargs: Extra options to send to the Gitlab server (e.g. sudo)
+            **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabDeleteError: If the server cannot perform the request
         """
-        if not isinstance(id, int):
-            id = id.replace('/', '%2F')
-        path = '%s/%s' % (self.path, id)
+        if id is None:
+            path = self.path
+        else:
+            if not isinstance(id, int):
+                id = id.replace('/', '%2F')
+            path = '%s/%s' % (self.path, id)
         self.gitlab.http_delete(path, **kwargs)
 
 
@@ -329,6 +388,23 @@ class ObjectDeleteMixin(object):
         self.manager.delete(self.get_id())
 
 
+class UserAgentDetailMixin(object):
+    @cli.register_custom_action(('Snippet', 'ProjectSnippet', 'ProjectIssue'))
+    @exc.on_http_error(exc.GitlabGetError)
+    def user_agent_detail(self, **kwargs):
+        """Get the user agent detail.
+
+        Args:
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabGetError: If the server cannot perform the request
+        """
+        path = '%s/%s/user_agent_detail' % (self.manager.path, self.get_id())
+        return self.manager.gitlab.http_get(path, **kwargs)
+
+
 class AccessRequestMixin(object):
     @cli.register_custom_action(('ProjectAccessRequest', 'GroupAccessRequest'),
                                 tuple(), ('access_level', ))
@@ -418,6 +494,11 @@ class TimeTrackingMixin(object):
             GitlabAuthenticationError: If authentication is not correct
             GitlabTimeTrackingError: If the time tracking update cannot be done
         """
+        # Use the existing time_stats attribute if it exist, otherwise make an
+        # API call
+        if 'time_stats' in self.attributes:
+            return self.attributes['time_stats']
+
         path = '%s/%s/time_stats' % (self.manager.path, self.get_id())
         return self.manager.gitlab.http_get(path, **kwargs)
 
@@ -486,3 +567,53 @@ class TimeTrackingMixin(object):
         """
         path = '%s/%s/reset_spent_time' % (self.manager.path, self.get_id())
         return self.manager.gitlab.http_post(path, **kwargs)
+
+
+class ParticipantsMixin(object):
+    @cli.register_custom_action(('ProjectMergeRequest', 'ProjectIssue'))
+    @exc.on_http_error(exc.GitlabListError)
+    def participants(self, **kwargs):
+        """List the participants.
+
+        Args:
+            all (bool): If True, return all the items, without pagination
+            per_page (int): Number of items to retrieve per request
+            page (int): ID of the page to return (starts with page 1)
+            as_list (bool): If set to False and no pagination option is
+                defined, return a generator instead of a list
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabListError: If the list could not be retrieved
+
+        Returns:
+            RESTObjectList: The list of participants
+        """
+
+        path = '%s/%s/participants' % (self.manager.path, self.get_id())
+        return self.manager.gitlab.http_get(path, **kwargs)
+
+
+class BadgeRenderMixin(object):
+    @cli.register_custom_action(('GroupBadgeManager', 'ProjectBadgeManager'),
+                                ('link_url', 'image_url'))
+    @exc.on_http_error(exc.GitlabRenderError)
+    def render(self, link_url, image_url, **kwargs):
+        """Preview link_url and image_url after interpolation.
+
+        Args:
+            link_url (str): URL of the badge link
+            image_url (str): URL of the badge image
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabRenderError: If the rendering failed
+
+        Returns:
+            dict: The rendering properties
+        """
+        path = '%s/render' % self.path
+        data = {'link_url': link_url, 'image_url': image_url}
+        return self.gitlab.http_get(path, data, **kwargs)
