@@ -26,7 +26,7 @@ fatal() { error "$@"; exit 1; }
 try() { "$@" || fatal "'$@' failed"; }
 
 NOVENV=
-PY_VER=2
+PY_VER=3
 API_VER=4
 while getopts :np:a: opt "$@"; do
     case $opt in
@@ -41,19 +41,18 @@ done
 
 case $PY_VER in
     2) VENV_CMD=virtualenv;;
-    3) VENV_CMD=pyvenv;;
+    3) VENV_CMD="python3 -m venv";;
     *) fatal "Wrong python version (2 or 3)";;
 esac
 
 case $API_VER in
-    3|4) ;;
-    *) fatal "Wrong API version (3 or 4)";;
+    4) ;;
+    *) fatal "Wrong API version (4 only)";;
 esac
 
 for req in \
     curl \
     docker \
-    "${VENV_CMD}" \
     ;
 do
     command -v "${req}" >/dev/null 2>&1 || fatal "${req} is required"
@@ -96,7 +95,7 @@ testcase() {
 
 if [ -z "$NOVENV" ]; then
     log "Creating Python virtualenv..."
-    try "$VENV_CMD" "$VENV"
+    try $VENV_CMD "$VENV"
     . "$VENV"/bin/activate || fatal "failed to activate Python virtual environment"
 
     log "Installing dependencies into virtualenv..."
@@ -118,7 +117,7 @@ while :; do
     curl -s http://localhost:8080/users/sign_in 2>/dev/null \
         | grep -q "GitLab Community Edition" && break
     I=$((I+5))
-    [ "$I" -lt 120 ] || fatal "timed out"
+    [ "$I" -lt 180 ] || fatal "timed out"
 done
 
 # Get the token
@@ -139,6 +138,6 @@ log "Config file content ($CONFIG):"
 log <$CONFIG
 
 log "Pausing to give GitLab some time to finish starting up..."
-sleep 30
+sleep 60
 
 log "Test environment initialized."
